<?php

namespace EventPortal\AuthPassword\Http\Controllers;

use Carbon\Carbon;
use EventPortal\AuthPassword\Http\Requests\StoreCreatePassword;
use EventPortal\AuthPassword\Rules\NovoRule;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Facades\Auth;
use Inertia\Inertia;
use Inertia\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class CreatePasswordController
{
    public function index(string $uuid): Response
    {
        throw_if($this->hasInvalidToken($uuid), NotFoundHttpException::class);

        $rules = collect(config('auth-password.validation.password'))
            ->reject(function ($rule) {
                return gettype($rule) === 'string';
            })
            ->map(function (NovoRule $rule) {
                return [
                    'label' => $rule->frontEndLabel(),
                    'validation' => $rule->frontEndValidation(),
                ];
            })
            ->values();

        return Inertia::render('CreatePassword/Index', [
            'token' => request()->token,
            'rules' => $rules,
        ]);
    }

    public function store(StoreCreatePassword $request, string $uuid): RedirectResponse
    {
        throw_if($this->hasInvalidToken($uuid), NotFoundHttpException::class);

        $user = config('auth.providers.users.model')::where('password_token', $uuid)->firstOrFail();

        $user->password = bcrypt($request->password);
        $user->password_created_at = now();
        $user->password_token = null;
        $user->save();

        Auth::login($user);

        return redirect($user->getHomeRoute());
    }

    private function hasInvalidToken(string $token): bool
    {
        $hasExpiryTime = config('auth-password.password_token_expires_in_minutes');

        if ($hasExpiryTime === null) {
            return false;
        }

        $user = config('auth.providers.users.model')::where('password_token', $token)->firstOrFail();

        return Carbon::parse($user->password_token_expires_at)->gte(now());
    }
}
